@@ -575,6 +575,11 @@ extract_pdf (const gchar          *uri,
 
 			tracker_sparql_builder_object_blank_close (metadata); /* GeoLocation */
 		}
+
+                if (xd->regions) {
+	                tracker_xmp_apply_regions (preupdate, metadata, NULL, xd);
+                }
+
 		tracker_xmp_free (xd);
 	} else {
 		/* So if we are here we have NO XMP data and we just
